import unittest

from .test_api import TestDesigner, TestConnection
from .test_utils import TestUtils, TestPlatform

__all__ = [
    'TestDesigner',
    'TestConnection',
    'TestUtils',
    'TestPlatform'
]

if __name__ == '__main__':
    unittest.main()



"""
Пакет для автоматизации взаимодействия с пакетным режимом 1С.
"""
__author__ = 'Alexander_NiMo'
__maintainer__ = __author__


__email__ = 'binipox@gmail.com'
__license__ = 'MIT'
__version__ = '0.1.4'


__all__ = (
    '__author__',
    '__email__',
    '__license__',
    '__maintainer__',
    '__version__',
)


from designer_cmd.api import Connection, Designer, RepositoryConnection, convert_cf_to_xml
import unittest
import os.path as path
import os
from designer_cmd.utils.utils import clear_folder


class TestConnection(unittest.TestCase):

    def setUp(self):
        pass

    def test_init_base_path(self):
        with self.assertRaises(AttributeError) as cm:
            con = Connection(user='user')
        the_exception = cm.exception
        self.assertEqual(
            the_exception.args[0],
            'Для соедеинения не определен путь к базе!',
            'Полученно неожиданное исключение'
        )

    def test_get_connection_params_file(self):

        loc_path = 'path'

        con = Connection(user='user', password='pass', file_path=loc_path)
        self.assertEqual(
            con.get_connection_params(),
            [f'/F', f'{path.abspath(loc_path)}', '/N', f'user', '/P', f'pass'],
            'Не верно определена строка подключени для файловой базы'
        )

    def test_get_connection_params_server(self):

        con = Connection(user='user', password='pass', server_path='192.168.1.1', server_base_ref='test')
        self.assertEqual(
            con.get_connection_params(),
            ['/S', '192.168.1.1\\test', '/N', 'user', '/P', 'pass'],
            'Не верно определена строка подключени для серверной базы'
        )

    def test_get_connection_string_file(self):

        loc_path = 'path'

        con = Connection(user='user', password='pass', file_path=loc_path)
        self.assertEqual(
            con.get_connection_string(),
            f'Usr=user Pwd=pass File={path.abspath(loc_path)}',
            'Не верно определена строка подключени для файловой базы'
        )


class TestDesigner(unittest.TestCase):

    def setUp(self):
        test_data_dir = path.join(path.dirname(__file__), 'test_data')

        self.temp_path = path.join(test_data_dir, 'temp')
        self.test_base_path = path.join(test_data_dir, 'base')
        self.cf_path = path.join(test_data_dir, '1Cv8.cf')
        self.dt_path = path.join(test_data_dir, '1Cv8.dt')
        self.cf_increment_path = path.join(test_data_dir, '1Cv8_increment.cf')

        self.cfe_path1 = path.join(test_data_dir, 'test.cfe')
        self.cfe_path2 = path.join(test_data_dir, 'test2.cfe')

        self.repo_obj_list = path.join(test_data_dir, 'obj_list')
        self.repo_obj_list_all = path.join(test_data_dir, 'obj_list_all_objects')
        self.merge_settings = path.join(test_data_dir, 'MergeSettings.xml')

        self.repo_user = 'user'
        self.repo_password = 'password'

        self.conn = self.db_connection()
        self.designer = Designer('', self.conn)

        clear_folder(self.test_base_path)
        clear_folder(self.temp_path)

    def db_connection(self):
        return Connection(file_path=self.test_base_path)

    def prepare_base(self):
        self.designer.create_base()
        self.designer.load_config_from_file(self.cf_path)
        self.designer.update_db_config()

    def prepare_repo(self) -> str:
        self.designer.create_base()
        repo_path = path.join(self.temp_path, 'repo')
        if not path.exists(repo_path):
            os.mkdir(repo_path)
        clear_folder(repo_path)
        self.designer.repo_connection = RepositoryConnection(repo_path, self.repo_user, self.repo_password)
        self.designer.create_repository()

        return repo_path

    def test_load_create_db(self):
        self.designer.create_base()

    def test_load_conf_from_file(self):
        self.prepare_base()

    def test_update_cfg(self):
        self.designer.create_base()
        self.designer.load_config_from_file(self.cf_path)
        self.designer.update_db_config()

    def test_dump_conf_to_file(self):
        self.prepare_base()
        cf_file_path = path.join(self.temp_path, '1Cv81.cf')
        self.designer.dump_config_to_file(cf_file_path)

        self.assertTrue(os.path.exists(cf_file_path), 'Выгрузка кофигурации не создана!')

    def test_dump_config_to_files(self):
        self.prepare_base()
        dir_xml_config_path = self.dump_to_files()

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'Не обнаружена выгрузка в xml'
        )

    def test_dump_config_to_file_increment(self):
        self.prepare_base()

        dir_xml_config_path = self.dump_to_files()

        cur_catalog_file_time = os.path.getatime(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml'))

        self.designer.load_config_from_file(self.cf_path)
        self.designer.update_db_config()
        self.designer.dump_config_to_files(dir_xml_config_path)

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник2.xml')),
            'Выгрузка не была выполнена.'
        )

        self.assertEqual(
            cur_catalog_file_time,
            os.path.getatime(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'Выгрузка не была инкриментальной'
        )

    def test_dump_extension_to_file(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        cf_file_path = path.join(self.temp_path, 'test_.cfe')
        self.designer.dump_extension_to_file(cf_file_path, 'test')

        self.assertTrue(
            os.path.exists(cf_file_path),
            'Не обнаружена выгрузка расширения'
        )

    def export_all_extension(self):
        cfe_dir_path = path.join(self.temp_path, 'cfe_dir')
        if not path.exists(cfe_dir_path):
            os.mkdir(cfe_dir_path)
        clear_folder(cfe_dir_path)

        self.designer.dump_extensions_to_files(cfe_dir_path)

        return cfe_dir_path

    def test_dump_extensions_to_files(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        cfe_dir_path = self.export_all_extension()

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test')),
            'Выгрузка не сформирована.'
        )

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test2')),
            'Выгрузка не сформирована.'
        )

    def test_load_extension(self):
        self.prepare_base()
        self.designer.load_extension_from_file(self.cfe_path1, 'test')
        self.designer.load_extension_from_file(self.cfe_path2, 'test2')

        cfe_dir_path = self.export_all_extension()

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test')),
            'Отсутствует расширение test.'
        )

        self.assertTrue(
            os.path.exists(os.path.join(cfe_dir_path, 'test2')),
            'Отсутствует расширение test2.'
        )

    def test_load_extension_from_files(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        cfe_dir_path = path.join(self.temp_path, 'cfe_dir', 'test')
        self.designer.dump_extension_to_files(cfe_dir_path, 'test')

        self.assertTrue(
            os.path.exists(cfe_dir_path),
            'Выгрузка не сформирована.'
        )

        self.designer.load_extension_from_files(cfe_dir_path, 'test')

    def test_add_repository(self):
        repo_path = self.prepare_repo()
        self.assertTrue(os.path.exists(os.path.join(repo_path, 'cache')), 'Хранилище не создано.')

    def test_add_user_to_repo(self):
        self.prepare_repo()
        self.designer.add_user_to_repository('user1', 'password1', rights='LockObjects')

    def test_lock_obj_in_repository(self):
        self.prepare_repo()
        self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)

    def test_unlock_obj_in_repository(self):
        self.prepare_repo()
        self.designer.unlock_objects_in_repository(self.repo_obj_list_all)

    def test_commit_repo(self):

        comment = 'test_test_test_1_2_3_4_5'

        self.prepare_repo()
        self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)
        self.designer.merge_config_with_file(self.cf_increment_path, self.merge_settings)
        self.designer.update_db_config()
        self.designer.commit_config_to_repo(comment, self.repo_obj_list_all)

        report_path = path.join(self.temp_path, 'repo_report.xml')
        self.designer.get_repo_report(report_path)

        with open(report_path, 'r', encoding='utf-8') as f:
            data = f.read()
            self.assertTrue(
                comment in data,
                'Commit не создан.'
            )

    def test_dump_config_from_repo(self):
        self.test_commit_repo()
        cf_file_path = path.join(self.temp_path, '1Cv81_repo.cf')
        self.designer.dump_config_to_file_from_repo(cf_file_path)
        self.assertTrue(os.path.exists(cf_file_path), 'Выгрузка кофигурации не создана!')

        self.check_common_module()

    def test_update_conf_from_repo(self):
        self.test_commit_repo()
        self.designer.load_db_from_file(self.dt_path)
        self.designer.update_conf_from_repo()
        self.check_common_module()

    def test_get_repo_report(self):
        self.prepare_repo()

        report_path = path.join(self.temp_path, 'repo_report.xml')
        self.designer.get_repo_report(report_path)

        self.assertTrue(
            os.path.exists(report_path),
            'Отчет не сформирован.'
        )

        with open(report_path, 'r', encoding='utf-8') as f:
            data = f.read()
            self.assertTrue(
                'Создание хранилища конфигурации' in data,
                'В отчете нет необходимых данных'
            )

    def test_encoding(self):
        self.designer.create_base()
        with self.assertRaises(SyntaxError) as ex:
            self.designer.create_base()
        self.assertEqual(ex.exception.msg, 'Не удалось выполнить команду!', 'Небыло вызвано исключение')

    def test_bind_cfg_to_repo(self):
        self.prepare_repo()
        self.designer.load_db_from_file(self.dt_path)

        self.designer.bind_cfg_to_repo()
        dir_xml_config_path = self.dump_to_files()

        self.assertFalse(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'База не была привязана.'
        )

    def test_merge_cf(self):
        self.prepare_base()
        self.designer.merge_config_with_file(self.cf_increment_path, self.merge_settings)

    def test_compare_conf(self):
        self.prepare_base()
        report_path = os.path.join(self.temp_path, 'report.txt')
        self.designer.compare_config_with_file(self.cf_path, report_path)

        self.assertTrue(os.path.exists(report_path), 'Отчет о сравнении не создан!')

    def test_dump_db_to_file(self):
        self.prepare_base()
        new_dt_path = os.path.join(self.temp_path, '1c.dt')
        self.designer.dump_db_to_file(new_dt_path)

        self.assertTrue(os.path.exists(new_dt_path), 'Файл выгрузки не создан!')

    def test_load_db_from_file(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

        dir_xml_config_path = self.dump_to_files()

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'Выгрузка не была выполнена.'
        )

    def test_unbind_cfg_from_repo(self):
        self.prepare_repo()
        new_user = 'new_user'
        new_passwd = 'passwd'
        self.designer.add_user_to_repository(new_user, new_passwd, 'LockObjects')

        self.designer.unbind_cfg_from_repo(True, True)

        self.designer.repo_connection.user = new_user
        self.designer.repo_connection.password = new_passwd

        self.designer.bind_cfg_to_repo()

        self.designer.repo_connection.user = self.repo_user
        self.designer.repo_connection.password = self.repo_password

        with self.assertRaises(SyntaxError) as er:
            self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)
        self.assertTrue(er.exception.args[0], 'Не удалось выполнить команду!')

        self.designer.repo_connection.user = new_user
        self.designer.repo_connection.password = new_passwd

        self.designer.lock_objects_in_repository(self.repo_obj_list_all, True)

    def check_common_module(self):
        dir_xml_config_path = self.dump_to_files()

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'CommonModules\\ОбщийМодуль1.xml')),
            'Объединение не выполнено.'
        )

    def dump_to_files(self) -> str:
        dir_xml_config_path = path.join(self.temp_path, 'xml_config')
        if not path.exists(dir_xml_config_path):
            os.mkdir(dir_xml_config_path)

        self.designer.dump_config_to_files(dir_xml_config_path)
        return dir_xml_config_path

    def tearDown(self):
        clear_folder(self.test_base_path)
        clear_folder(self.temp_path)



import logging
import sys
import os.path as path
import os
import subprocess
from functools import total_ordering
import shutil
from ctypes import windll


logger = logging.getLogger(__name__)


@total_ordering
class PlatformVersion:

    def __init__(self, version: str):
        self.version: str = version
        if self.version != '':
            version_weight = self.__get_version_weight()
        else:
            version_weight = 9999999999
        self.__version_weight: int = version_weight

    @property
    def version_weight(self) -> int:
        return self.__version_weight

    def __get_version_weight(self) -> int:
        """
        Вычисляет вес версии, для сравнения версий
        Подразумевается, что версии, для которых будет вычисляться октавы имеют одинаковые длины октавово
        и их длина не больше 4

        :param element:
        :return:
        """
        octs_first = self.version.split('.')

        while len(octs_first) < 4:
            octs_first.append('0')

        if len(octs_first) > 4:
            raise ValueError(f'Версия платформы имеет 4 октава, передано значение {octs_first}')

        octs_first.reverse()
        koef = 1000
        summ = 0
        for i, val in enumerate(octs_first):
            if len(val) > 4:
                logger.error(f'Длина октава {val} в версии {self.version} больше 4, вычисление веса не поддерживается.')
                raise ValueError('Ошибка вычисления веса версии, длинна октава не должна быть больше 4.')
            summ += int(val) * koef ** i
        return summ

    def __str__(self):
        return self.version

    def __repr__(self):
        return self.version

    def _is_valid_operand(self, other):
        return hasattr(other, "version_weight")

    def __eq__(self, other):
        if not self._is_valid_operand(other):
            return NotImplemented
        return self.version_weight == other.version_weight

    def __lt__(self, other):
        if not self._is_valid_operand(other):
            return NotImplemented
        return self.version_weight < other.version_weight


def windows_platform() -> bool:
    return 'win' in sys.platform


def get_platform_path(version: PlatformVersion) -> str:
    """
    Вычисляет путь к исполняемому файлу платформы

    :param version: Версия платформы.
    :return:
    """

    if windows_platform():
        platform_path = __get_platform_path_windows(version)
    else:
        platform_path = __get_platform_path_linux(version)

    return platform_path


def __get_version_path(dir_1c, version: PlatformVersion) -> str:
    bin_path = 'bin\\1cv8.exe'
    exept_dir = ['common', 'conf']

    versions = [PlatformVersion(dir_name) for dir_name in os.listdir(dir_1c) if dir_name not in exept_dir]

    if len(versions) == 0:
        return ''

    if version.version == '':
        # Получим максимальную версию
        return path.join(dir_1c, sorted(versions, key=lambda v: v.version_weight, reverse=True)[0].version, bin_path)
    elif version in versions:
        return path.join(dir_1c, version.version, bin_path)
    else:
        return ''


def __get_platform_path_windows(version: PlatformVersion) -> str:
    version_path = __get_version_path(path.join(os.getenv('ProgramW6432'), '1cv8'), version)

    if version_path == '':
        version_path = __get_version_path(path.join(os.getenv('ProgramFiles'), '1cv8'), version)

    if version_path == '':
        if version == '':
            logger.critical('Не обнаружена установленная 1с. Выполнение невозможно.')
            raise EnvironmentError('Не обнаружена установленная 1с.')
        else:
            logger.critical(f'Не обнаружена установленная версия 1с номер версии:{version}. Выполнение невозможно.')
            raise EnvironmentError(f'Не обнаружена версия {version} 1с.')

    return version_path


def __get_platform_path_linux(version: PlatformVersion) -> str:
    platform_path = ''

    return platform_path


def execute_command(command: str, params: list, timeout: int = None) -> tuple:
    """
    Выполняет команду в системе.

    :param command: Команда
    :param params: Параметры команды
    :param timeout: Лимит времени на выполнение команды, после выхода за пределы будет возбуждено исключение.
    :return:
    """
    if windows_platform():
        result = __execute_windows_command(command, params, timeout)
    else:
        result = __execute_linux_command(command, params, timeout)
    return result


def __execute_windows_command(command: str, params: list, timeout: int) -> tuple:
    """
    Выполняет команду системы в windows

    :param command:
    :return:
    """
    prev_codepage = windll.kernel32.GetConsoleOutputCP()
    windll.kernel32.SetConsoleOutputCP(65001)
    try:
        process = subprocess.run(
            args=[command] + params,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            check=True,
            encoding=encoding(),
            timeout=timeout
        )
        if process.returncode == 0:
            msg = process.stdout
        else:
            msg = process.stderr
        return process.returncode, msg.strip()
    except subprocess.TimeoutExpired:
        return 1, 'Выполнение процесса вышло за рамки отведенного времени.'
    except subprocess.CalledProcessError as e:
        return 1, f'Ошибка выполнения команды {e}'
    finally:
        windll.kernel32.SetConsoleOutputCP(prev_codepage)


def encoding() -> str:
    """
    Определяет кодировку в контексте системы.
        Спасибо (@vbondarevsky)
    :return:
    """
    if windows_platform():
        import ctypes
        return f"cp{ctypes.windll.kernel32.GetOEMCP()}"
    else:
        return (sys.stdout.encoding if sys.stdout.isatty() else
                sys.stderr.encoding if sys.stderr.isatty() else
                sys.getfilesystemencoding() or "utf-8")


def __execute_linux_command(command: str, params: list, timeout: int) -> str:
    """
    Выполняет команду системы в linux

    :param command:
    :return:
    """
    raise NotImplementedError('Не реализовано!')
    pass


def xml_conf_version_file_exists(dir_path: str):
    version_file_name = "ConfigDumpInfo.xml"
    test_path = os.path.join(dir_path, version_file_name)
    return os.path.exists(test_path)


def clear_folder(dir_path):
    if path.exists(dir_path):
        filelist = [f for f in os.listdir(dir_path)]
        for file_name in filelist:
            if '.gitkeep' in file_name:
                continue
            file_path = path.join(dir_path, file_name)
            if path.isdir(file_path):
                shutil.rmtree(file_path)
            else:
                os.remove(file_path)


from designer_cmd.utils import (get_platform_path, execute_command, xml_conf_version_file_exists,
                                PlatformVersion, clear_folder, windows_platform)
import os
import logging
import tempfile
from typing import Optional, Callable

logger = logging.getLogger(__name__)


class RepositoryConnection:

    def __init__(self, repository_path: str, user: str = '', password: str = ''):
        self.password = password
        self.user = user

        full_repo_path = repository_path
        if 'tcp' not in repository_path:
            full_repo_path = os.path.abspath(repository_path)

        self.repository_path = full_repo_path

    def get_connection_params(self) -> list:
        params = [f'/ConfigurationRepositoryF', f'{self.repository_path}']
        if self.user != '':
            params.append(f'/ConfigurationRepositoryN')
            params.append(f'{self.user}')
        if self.password != '':
            params.append(f'/ConfigurationRepositoryP')
            params.append(f'{self.password}')

        return params

    def __repr__(self):
        return f'path: {self.repository_path} user: {self.user}'


class Connection:
    """
    Описывает соединение с базой
    """
    def __init__(self, user: str= '',
                 password: str = '',
                 file_path: str = '',
                 server_path: str = '',
                 server_base_ref: str = '',
                 ib_name: str = '',
                 time_out: int = 3600):

        if file_path == '' and (server_path == '' or server_base_ref == '') and ib_name == '':
            raise AttributeError('Для соедеинения не определен путь к базе!')

        self.user = user
        self.password = password

        self.ib_name = ib_name

        self.file_path = os.path.abspath(file_path) if file_path != '' else ''
        self.server_path = server_path
        self.server_base_ref = server_base_ref

        self.__time_out = time_out

    @property
    def timeout(self):
        return self.__time_out

    @timeout.setter
    def timeout(self, value):
        self.__time_out = value

    def get_connection_params(self) -> list:
        """
        Возвращает строковое представление параметров для подключения по текущему описанию соединения

        :return: str : Текстовые параметры для командной строки
        """
        params = []
        if self.file_path != '':
            params += [f'/F', f'{self.file_path}']
        elif self.ib_name != '':
            params += [f'/IBName', f'{self.ib_name}']
        else:
            params += [f'/S', f'{self.server_path}\\{self.server_base_ref}']

        if self.user != '':
            params += [f'/N', f'{self.user}']
            params += [f'/P', f'{self.password}']

        return params

    def get_connection_string(self):
        if self.ib_name != '':
            raise ValueError('Не возможно сформировать строку подключения для базы из списка.')

        if self.file_path != '':
            if self.user == '':
                return f'File={self.file_path}'
            else:
                return f'Usr={self.user} Pwd={self.password} File={self.file_path}'
        else:
            raise NotImplementedError('Создание базы в серверном варианте не реализованно.')

    def __repr__(self):
        if self.file_path != '':
            connection_path = f'File: {self.file_path}'
        elif self.ib_name != '':
            connection_path = f'IBName: {self.ib_name}'
        else:
            connection_path = f'Server: {self.server_path} base_ref: {self.server_base_ref}'
        connection_path += f'user: {self.user}' if self.user != '' else ''
        return connection_path


def have_repo_connection(func):

    def warper(self, *args, **kwargs):
        if self.repo_connection is None:
            raise AttributeError('Не передано описание подключения к хранилищу!')
        return func(self, *args, **kwargs)

    return warper


class Designer:

    def __init__(self, platform_version: str, connection: Connection, repo_connection: RepositoryConnection = None):

        self.repo_connection = repo_connection
        self.platform_version: PlatformVersion = PlatformVersion(platform_version)
        self.connection = connection
        self.platform_path = get_platform_path(self.platform_version)

    def __execute_command(self, mode: str, command_params: list, connection_params_required: bool = True):
        params = [mode]
        if connection_params_required:
            params += self.connection.get_connection_params()
        params += command_params
        params += ['/DisableStartupDialogs /DisableStartupMessages']
        debug_file_name = self.add_debug_params(params)

        str_command = ' '.join(params)
        logger.debug(f'Выполняю команду {self.platform_path} {str_command}')

        result = execute_command(self.platform_path, params, self.connection.timeout)

        if result[0] != 0:
            try:
                f = open(debug_file_name, encoding='utf-8')
                error_text = f.read()
            except UnicodeDecodeError:
                f = open(debug_file_name, encoding='cp1251')
                error_text = f.read()
            f.close()

            logger.error(f'При выполнении команды произошла ошибка:\n'
                         f'{error_text}\n'
                         f'{result[1]}')
            os.remove(debug_file_name)
            raise SyntaxError('Не удалось выполнить команду!')
        os.remove(debug_file_name)

    def add_debug_params(self, params) -> str:
        debug_file_name = tempfile.mkstemp('.log')
        params.append('/Out')
        params.append(debug_file_name[1])
        os.close(debug_file_name[0])
        return debug_file_name[1]

    def create_base(self):
        """
        Создает базу данных.
        Соответствует режиму CREATEINFOBASE <строка соединения>

        """
        logger.info(f'Создаю базу по соединению: {self.connection}')
        params = [f'{self.connection.get_connection_string()}']
        self.__execute_command('CREATEINFOBASE', params, False)

    def manage_support(self):

        logger.info(f'Снимаю конфигурацию БД по соединению {self.connection} с поддержки')

        params = [f'/ManageCfgSupport', '-disableSupport', '-force']

        self.__execute_command(f'DESIGNER', params)

    def update_db_config(self, dynamic: bool = False, warnings_as_errors: bool = False, on_server: bool = False):
        """
        Обновляет конфигурацию db. (соответствует команде /UpdateDBCfg)

        :return:
        """
        logger.info(f'Обновляю конфигурацию БД по соединению {self.connection}')
        params = [f'/UpdateDBCfg']

        if dynamic:
            params.append('-Dynamic +')
        if warnings_as_errors:
            params.append('-WarningsAsErrors')
        if on_server:
            params.append('-Server')

        self.__execute_command(f'DESIGNER', params)

    def load_db_from_file(self, file_path: str) -> None:
        """
        Загрузка базы из файла dt (соответствует команде /RestoreIB)

        :param file_path: str - Путь к файлу базы (dt.)
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(f'Загружаю файл dt {full_file_path} в БД по соединению {self.connection}')
        params = ['/RestoreIB', f'{full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def dump_db_to_file(self, file_path: str) -> None:
        """
        Выгрузка базы в файл (соответствует команде /DumpIB)

        :param file_path: str - Путь к файлу для выгрзки базы (dt.)
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(f'Выгружаю файл dt по пути {full_file_path} из БД по соединению {self.connection}')
        params = ['/DumpIB', f'{full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def load_config_from_files(self, catalog_path: str, list_file: Optional[str] = None) -> None:
        """
        Загружает конфигурацию из файлов (соответствует команде /LoadConfigFromFiles)

        :param list_file: путь к файлу со списком файлов к загрузке
        :param catalog_path: str - путь к каталогу из которого необходимо произвести загрузку.
        :return:
        """
        full_catalog_path = os.path.abspath(catalog_path)
        logger.info(f'Загружаю конфигурацию из файлов {full_catalog_path} конфигурацию БД по соединению {self.connection}')
        params = [f'/LoadConfigFromFiles', f'{full_catalog_path}']

        if list_file is not None and os.path.exists(list_file):
            params.extend([f'-listFile', f'{os.path.abspath(list_file)}'])
            params.extend([f'-Format', 'Hierarchical'])

        self.__execute_command(f'DESIGNER', params)

    def dump_config_to_files(self, catalog_path: str, update: bool = True) -> None:
        """
        Выгружает конфигурацию в файлы (соответствует команде /DumpConfigToFiles)

        :param update:
        :param catalog_path: Каталог в который будет выгружен файл
        :return:
        """
        full_catalog_path = os.path.abspath(catalog_path)
        logger.info(
            f'Выгружаю конфигурацию в файлы {full_catalog_path} по соединению {self.connection}')
        params = [f'/DumpConfigToFiles', f'{full_catalog_path}']

        increment_platform_version = PlatformVersion('8.3.10')

        if (update and xml_conf_version_file_exists(catalog_path)
                and self.platform_version >= increment_platform_version):
            params.append('-update')
            params.append('-force')

        self.__execute_command(f'DESIGNER', params)

    def load_config_from_file(self, file_path: str) -> None:
        """
        Загружает конфигурацию в базу из файла cf (соответствует команде /LoadCfg)

        :param file_path:
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(
            f'Загружаю конфигурацию из файла {full_file_path} в конфигурацию БД по соединению {self.connection}')
        params = [f'/LoadCfg', f'{full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def dump_config_to_file(self, file_path: str) -> None:
        """
        Выполнить сохранение конфигурации в файл cf (соответствует команде /DumpCfg)
        :param file_path:
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(
            f'Сохраняю конфигурацию в файл {full_file_path} из конфигурации БД по соединению {self.connection}')
        params = [f'/DumpCfg', f'{full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def dump_extension_to_file(self, file_path: str, extension_name: str):
        full_file_path = os.path.abspath(file_path)
        logger.info(
            f'Сохраняю расширения в файл {full_file_path} из конфигурации БД по соединению {self.connection}')
        params = [
            f'/DumpConfigToFiles', f'{full_file_path}',
            f'-Extension', f'{extension_name}',
            '-force'
        ]

        self.__execute_command(f'DESIGNER', params)

    def dump_extension_to_files(self, dir_path: str, extansion_name: str):
        full_dir_path = os.path.abspath(dir_path)
        logger.info(
            f'Сохраняю расширения в файл {full_dir_path} из конфигурации БД по соединению {self.connection}')
        params = [
            f'/DumpConfigToFiles', f'{full_dir_path}',
            f'-Extension', f'{extansion_name}'
        ]
        self.__execute_command(f'DESIGNER', params)

    def dump_extensions_to_files(self, dir_path):
        full_dir_path = os.path.abspath(dir_path)
        logger.info(
            f'Сохраняю расширения в файл {full_dir_path} из конфигурации БД по соединению {self.connection}')
        params = [
            f'/DumpConfigToFiles', f'{full_dir_path}',
            f'-AllExtensions',
        ]
        self.__execute_command(f'DESIGNER', params)

    def load_extension_from_file(self, extension_file_path: str, name: str):
        """
        Выполняет загрузку расширения из файла cfe в базу
        :param extension_file_path:
        :param name:
        """
        full_file_path = os.path.abspath(extension_file_path)
        logger.info(
            f'Загружаю расширения из файла {full_file_path} в конфигурацию БД по соединению {self.connection}')
        params = [
            f'/LoadCfg', f'{full_file_path}',
            f'-Extension', f'{name}'
        ]
        self.__execute_command(f'DESIGNER', params)

    def load_extension_from_files(self, extension_folder: str, name: str):
        """
        Выполняет загрузку расширения из каталога в базу
        :param extension_folder:
        :param name:
        """
        full_catalog_path = os.path.abspath(extension_folder)
        logger.info(
            f'Загружаю расширение c именем {name} из файлов {full_catalog_path} конфигурацию '
            f'БД по соединению {self.connection}')
        params = [
            f'/LoadConfigFromFiles', f'{full_catalog_path}',
            f'-Extension', f'{name}'
        ]

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def create_repository(self):

        logger.info(
            f'Создание хранилища по подключению {self.repo_connection}'
            f'БД по соединению {self.connection}')

        params = self.repo_connection.get_connection_params()
        params.append(f'/ConfigurationRepositoryCreate')
        params.append('-AllowConfigurationChanges')
        params.append('-ChangesAllowedRule')
        params.append('ObjectIsEditableSupportEnabled')
        params.append('-ChangesNotRecommendedRule')
        params.append('ObjectIsEditableSupportEnabled')

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def add_user_to_repository(self, user: str, password: str = '', rights: Optional[str] = None ):
        """
        Добавляет пользователя в хранилище (ConfigurationRepositoryAddUser)

        :type password: str
        :type user: str
        :type rights: str
        :param user: Пользователь хранилища
        :param password: Пароль к хранилищу
        :param rights: Роль пользователя возможные занчения:
            ReadOnly ‑ право на просмотр;
            LockObjects ‑ право на захват объектов;
            ManageConfigurationVersions ‑ право на изменение состава версий;
            Administration ‑ право на административные функции;
        """
        logger.info(
            f'добавлеине пользователя в хранилище {self.repo_connection}'
            f'из БД по соединению {self.connection}')

        params = self.repo_connection.get_connection_params()
        params.append(f'/ConfigurationRepositoryAddUser')
        params.extend([
            '-User', f'{user}',
            '-Pwd', f'{password}',
        ])
        if rights is None:
            rights = 'ReadOnly'

        params.extend([
            '-Rights', f'{rights}'
        ])

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def unlock_objects_in_repository(self, objects_list: str):

        file_path = os.path.abspath(objects_list)
        logger.info(
            f'Отправляю объекты в хранилище {self.repo_connection} по списку объектов из файла {file_path}'
            f'БД по соединению {self.connection}')
        params = self.repo_connection.get_connection_params()
        params.extend([
            f'/ConfigurationRepositoryUnLock',
            f'-Objects', f'{file_path}'
        ])
        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def lock_objects_in_repository(self, objects: str, force: bool = False):

        file_path = os.path.abspath(objects)
        logger.info(
            f'Захватываю объекты в хранилище {self.repo_connection} по списку объектов из файла {file_path}'
            f'БД по соединению {self.connection}')
        params = self.repo_connection.get_connection_params()
        params.extend([
            f'/ConfigurationRepositoryLock',
            f'-Objects', f'{file_path}'
        ])
        if force:
            params.append('-revised')

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def commit_config_to_repo(self, comment: str = '', objects: Optional[str] = None):
        """
        Выполняет отправку объектов в хранилище. операция /ConfigurationRepositoryCommit
        """

        logger.info(
            f'Отправка объектов в хранилище {self.repo_connection}'
            f'БД по соединению {self.connection}')

        params = self.repo_connection.get_connection_params()
        params.append(f'/ConfigurationRepositoryCommit')

        if objects is not None:
            params.extend([
                '-Objects', f'{objects}'
            ])
        params.extend([
            '-comment', f'{comment}'
        ])
        params.append('-force')

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def dump_config_to_file_from_repo(self, file_path, version: Optional[str] = None):
        """
        Выполняет выгрузку конфигурации из хранилища /ConfigurationRepositoryDumpCfg
        """
        full_file_path = os.path.abspath(file_path)

        logger.info(
            f'Выгрузка cf из хранилища {self.repo_connection}'
            f'БД по соединению {self.connection}')

        params = self.repo_connection.get_connection_params()
        params.append(f'/ConfigurationRepositoryDumpCfg')
        params.append(f'{full_file_path}')

        cfg_version = -1
        if version is not None:
            cfg_version = version
        params.extend([
            '-v', f'{cfg_version}'
        ])

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def update_conf_from_repo(self, version: Optional[int] = None):
        """
        Выполняет обновление конфигурации из хранилища /ConfigurationRepositoryUpdateCfg
        """
        logger.info(
            f'Обновление конфигурации БД из хранилища {self.repo_connection}'
            f'БД по соединению {self.connection}')

        params = self.repo_connection.get_connection_params()
        params.append(f'/ConfigurationRepositoryUpdateCfg')

        cfg_version = -1
        if version is not None:
            cfg_version = version
        params.extend([
            '-v', f'{cfg_version}'
        ])

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def bind_cfg_to_repo(self):
        """
        Выполняет привязку базы к хранилищу /ConfigurationRepositoryBindCfg
        """
        logger.info(
            f'Привязка базы к хранилищу {self.repo_connection}'
            f'БД по соединению {self.connection}')

        params = self.repo_connection.get_connection_params()
        params.append(f'/ConfigurationRepositoryBindCfg')
        params.append('-forceBindAlreadyBindedUser')
        params.append('-forceReplaceCfg')

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def unbind_cfg_from_repo(self, force: bool = False, local: bool = False):
        """
        Отвязывает базу от хранилища /ConfigurationRepositoryBindCfg
        :param force: выполняет подключение даже в том случае, если для данного пользователя уже есть конфигурация,
            связанная с данным хранилищем
        :param local: использовать параметры репозитория (локальная отвязка конфигурации)
        """
        if local:
            params = self.repo_connection.get_connection_params()
        else:
            params = list()

        params.append(f'/ConfigurationRepositoryUnbindCfg')
        if force:
            params.append('-force')

        self.__execute_command(f'DESIGNER', params)

    @have_repo_connection
    def get_repo_report(self,
                        report_file: str,
                        v_begin: Optional[int] = None,
                        v_end: Optional[int] = None,
                        group_by_obj: bool = False,
                        group_by_comment: bool = False):

        full_report_file = os.path.abspath(report_file)

        logger.info(
            f'Формирование отчета в файл {full_report_file} из хранилища {self.repo_connection}'
            f'БД по соединению {self.connection}')

        params = self.repo_connection.get_connection_params()
        params.append(f'/ConfigurationRepositoryReport')
        params.append(f'{full_report_file}')

        if v_begin is not None:
            params.extend([
                '-NBegin', f'{v_begin}'
            ])

        if v_begin is not None:
            params.extend([
                '-NEnd', f'{v_end}'
            ])

        if group_by_obj:
            params.append('-GroupByObject')

        if group_by_comment is not None:
            params.append('-GroupByComment')

        self.__execute_command(f'DESIGNER', params)

    def merge_config_with_file(self, cf_file_path: str, settings_path: str) -> None:
        """
        Выполнить объединение текущей конфигурации с файлом (с использованием файла настроек).
        (соответствует команде /MergeCfg)

        :param cf_file_path: str : Путь к файлу cf для объединения с текущей конфой
        :param settings_path: str : Путь к файлу настроек объединения
        :return:
        """
        full_cf_file_path = os.path.abspath(cf_file_path)
        full_path_settings_path = os.path.abspath(settings_path)
        logger.info(
            f'Объединение конфигурации по соединению {self.connection} с файлом {full_cf_file_path} '
            f'по настройкам из файла {full_path_settings_path}')
        params = [
            f'/MergeCfg', f'{full_cf_file_path}',
            f'-Settings', f'{full_path_settings_path}'
        ]
        self.__execute_command(f'DESIGNER', params)

    def compare_config_with_file(self, cf_file_path: str, report_path: str) -> None:
        """
        Выполнить сравнение двух конфигураций и сформировать файл с отчетом о сравнении.
        (соответствует команде /CompareCfg)
        :param cf_file_path: Путь к файлу cf, который необходимо сравнить с конфигурацией.
        :param: report_path: Путь к файлу, в который необходимо сохранить отчет.
        :return:
        """
        full_cf_file_path = os.path.abspath(cf_file_path)
        full_report_path = os.path.abspath(report_path)
        logger.info(
            f'Сравниваю конфигурацию по соединению {self.connection} с файлом {full_cf_file_path} '
            f'для сохранения отчета по пути {full_report_path} ')
        params = [
            '/CompareCfg',
            '-FirstConfigurationType',
            'MainConfiguration',
            '-SecondConfigurationType',
            'File',
            '-SecondFile',
            f'{full_cf_file_path}',
            '-IncludeChangedObjects',
            '-IncludeDeletedObjects',
            '-IncludeAddedObjects',
            '–ReportType',
            'txt',
            '-ReportFormat',
            'Full',
            '-ReportFile',
            f'{full_report_path}'
        ]
        self.__execute_command(f'DESIGNER', params)


def convert_cf_to_xml(
        cf_file_path: str,
        platform_version: str = '',
        out_path: Optional[str] = None,
        temp_path: Optional[str] = None,
        clear_temp_folder: bool = True) -> str:

    def convert_function(designer, full_cf_path, out_path):
        designer.load_config_from_file(full_cf_path)
        designer.dump_config_to_files(out_path)

    return _convert_to_xml(cf_file_path, convert_function, platform_version, out_path, temp_path, clear_temp_folder)


def convert_cfe_to_xml(
        cf_file_path: str,
        platform_version: str = '',
        out_path: Optional[str] = None,
        temp_path: Optional[str] = None,
        clear_temp_folder: bool = True) -> str:

    def convert_function(designer, full_cf_path, out_path):
        designer.load_extension_from_file(full_cf_path)
        designer.dump_extensions_to_files(out_path)

    return _convert_to_xml(cf_file_path, convert_function, platform_version, out_path, temp_path, clear_temp_folder)


def _convert_to_xml(
        file_path: str,
        function: Callable[[Designer, str, str], None],
        platform_version: str = '',
        out_path: Optional[str] = None,
        temp_path: Optional[str] = None,
        clear_temp_folder: bool = True) -> str:

    full_cf_path = os.path.abspath(file_path)

    if out_path is None:
        out_path = os.path.join(
            os.path.basename(full_cf_path),
            'cf'
        )
    if not os.path.exists(out_path):
        os.mkdir(out_path)

    rm_dir = False
    if temp_path is None:
        temp_path = tempfile.mkdtemp()
        rm_dir = True
    _temp_path = os.path.join(temp_path, 'base')
    os.mkdir(_temp_path)

    connection = Connection('', '', _temp_path)
    designer = Designer(platform_version, connection)

    designer.create_base()

    function(designer, full_cf_path, out_path)

    if clear_temp_folder:
        if rm_dir:
            clear_folder(temp_path)
            os.rmdir(temp_path)
        else:
            clear_folder(temp_path)

    return out_path


from designer_cmd.utils import utils

import unittest
from unittest import mock


class TestPlatform(unittest.TestCase):

    def test_get_version_weight(self):

        platform = utils.PlatformVersion('8.3.14.1232')
        weight = platform.version_weight
        self.assertEqual(weight, 8003015232)

    def test_get_version_weight_eseption(self):
        with self.assertRaises(ValueError) as error:
            utils.PlatformVersion('8.3.14.123212')
        self.assertEqual(
            error.exception.args[0],
            'Ошибка вычисления веса версии, длинна октава не должна быть больше 4.',
            'Проверка на ошибочную версию не прошла.'
        )

    @mock.patch('os.listdir')
    def test_get_platform_path_exeption(self, os_listdir):

        check_version = utils.PlatformVersion('8.3.14.1232')
        os_listdir.return_value = [
            '8.3.11.1232',
            '8.3.12.1132',
            '8.3.14.1231',
            '8.2.14.1232',
            '8.3.15.1232',
        ]

        with self.assertRaises(EnvironmentError) as error:
            utils.get_platform_path(check_version)
        self.assertEqual(
            error.exception.args[0],
            f'Не обнаружена версия {check_version} 1с.',
            'Проверка на ошибку поиска не пройдена'
        )

    @mock.patch('os.listdir')
    def test_get_platform_path(self, os_listdir):
        pref_path = 'PATH'
        max_version = utils.PlatformVersion('8.3.17.1212')
        os_listdir.return_value = [
            '8.3.11.1232',
            '8.3.12.1132',
            '8.3.14.1231',
            '8.2.14.1232',
            '8.3.15.1232',
            max_version.version
        ]
        check_version = utils.PlatformVersion('8.2.14.1232')
        with mock.patch('os.getenv') as os_getenv:
            os_getenv.return_value = pref_path
            platform_path = utils.get_platform_path(check_version)
            self.assertEqual(
                platform_path,
                f'{pref_path}\\1cv8\\{check_version}\\bin\\1cv8.exe',
                'Не прошла проверка не получение версии'
            )

            platform_path = utils.get_platform_path(utils.PlatformVersion(''))
            self.assertEqual(
                platform_path,
                f'{pref_path}\\1cv8\\{max_version}\\bin\\1cv8.exe',
                'Не прошла проверка на получение последней версии'
            )

    def test_platform_eq_operators(self):

        vresion_max = utils.PlatformVersion('')
        versuon_1 = utils.PlatformVersion('8.3.14.1522')
        version_2 = utils.PlatformVersion('8.3.14.1422')
        versuon_3 = utils.PlatformVersion('8.3.14.1522')

        self.assertGreater(vresion_max, versuon_1, 'Проверка на сравление с максимальной версией не прошла.')
        self.assertGreater(vresion_max, version_2, 'Проверка на сравление с максимальной версией не прошла.')
        self.assertGreater(versuon_1, version_2, 'Проверка на сравление версий не прошла.')
        self.assertEqual(versuon_1, versuon_3, 'Проверка на равенство не прошла')
        self.assertNotEqual(versuon_1, version_2, 'Провенка на неравенство не прошла.')


class TestUtils(unittest.TestCase):

    def setUp(self):
        pass

    def test_execute_command(self):
        result = utils.execute_command('dir', [])
        self.assertEqual(result[0], 0, 'Не удалось выполнить команду систему!')

        with self.assertRaises(FileNotFoundError) as error:
            utils.execute_command('dir111', [])
        self.assertEqual(
            error.exception.args[0],
            2,
            'Проверка на ошибочную команду провалилась!'
        )

    def test_timout_exception(self):
        result = utils.execute_command('cmd.exe', ['/c', 'pause 10'], 1)
        self.assertTrue(result[0] == 1, 'Ошибка по таймауту не произошла')

    def tearDown(self):
        pass



import os
from importlib.machinery import SourceFileLoader

from pkg_resources import parse_requirements
from setuptools import find_packages, setup

module_name = 'designer_cmd'

module = SourceFileLoader(
    module_name, os.path.join(module_name, '__init__.py')
).load_module()


def load_requirements(fname: str) -> list:
    requirements = []
    with open(fname, 'r') as fp:
        for req in parse_requirements(fp.read()):
            extras = '[{}]'.format(','.join(req.extras)) if req.extras else ''
            requirements.append(
                '{}{}{}'.format(req.name, extras, req.specifier)
            )
    return requirements


with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name=module_name,
    version=module.__version__,
    author=module.__author__,
    author_email=module.__email__,
    license=module.__license__,
    description='Пакет для автоматизации взаимодействия с пакетным режимом 1С.',
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/AlexanderNiMo/designer_cmd',
    platforms='Windows',
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: Russian',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython'
    ],
    python_requires='>=3.6',
    packages=find_packages(exclude=['tests']),
    install_requires=load_requirements('requirements.txt'),
    include_package_data=True,
    test_suite='designer_cmd.tests',
)
